@@ -30,6 +30,7 @@
 #include "pkcs11/pkcs11g.h"
 #include "pkcs11/pkcs11i.h"
 
+#include "egg/egg-error.h"
 #include "egg/egg-unix-credentials.h"
 
 #include <sys/types.h>
@@ -2223,7 +2224,7 @@ gck_rpc_layer_accept (void)
 	
 	ds->thread = g_thread_create (run_dispatch_thread, &(ds->socket), TRUE, &error);
 	if (!ds->thread) {
-		gck_rpc_warn ("couldn't start thread: %s", error && error->message ? error->message : "");
+		gck_rpc_warn ("couldn't start thread: %s", egg_error_message (error));
 		close (new_fd);
 		free (ds);
 		return;
@@ -2264,8 +2265,11 @@ gck_rpc_layer_uninitialize (void)
 		pkcs11_socket_path = NULL;
 	}
 
+	ds = pkcs11_dispatchers;
+	pkcs11_dispatchers = NULL;
+
 	/* Stop all of the dispatch threads */
-	for (ds = pkcs11_dispatchers; ds; ds = next) {
+	for (; ds; ds = next) {
 		next = ds->next;
 
 		/* Forcibly shutdown the connection */
@@ -2350,8 +2354,11 @@ gck_rpc_layer_shutdown (void)
 		pkcs11_socket_path = NULL;
 	}
 
+	ds = pkcs11_dispatchers;
+	pkcs11_dispatchers = NULL;
+
 	/* Stop all of the dispatch threads */
-	for (ds = pkcs11_dispatchers; ds; ds = next) {
+	for (; ds; ds = next) {
 		next = ds->next;
 		
 		/* Forcibly shutdown the connection */
