@@ -27,6 +27,8 @@
 #include "gck-secret-item.h"
 #include "gck-secret-textual.h"
 
+#include "egg/egg-error.h"
+
 #include "gck/gck-attributes.h"
 #include "gck/gck-credential.h"
 #include "gck/gck-secret.h"
@@ -48,6 +50,7 @@ struct _GckSecretCollection {
 	GHashTable *items;
 	gchar *filename;
 	guint32 watermark;
+	GArray *template;
 };
 
 G_DEFINE_TYPE (GckSecretCollection, gck_secret_collection, GCK_TYPE_SECRET_OBJECT);
@@ -72,7 +75,7 @@ load_collection_and_secret_data (GckSecretCollection *self, GckSecretData *sdata
 	/* Read in the keyring */
 	if (!g_file_get_contents (path, (gchar**)&data, &n_data, &error)) {
 		g_message ("problem reading keyring: %s: %s",
-		           path, error && error->message ? error->message : "");
+		           path, egg_error_message (error));
 		g_clear_error (&error);
 		return GCK_DATA_FAILURE;
 	}
@@ -108,7 +111,7 @@ find_unlocked_credential (GckCredential *cred, GckObject *object, gpointer user_
 
 	g_return_val_if_fail (!*result, FALSE);
 
-	if (gck_credential_get_data (cred)) {
+	if (gck_credential_peek_data (cred, GCK_TYPE_SECRET_DATA)) {
 		*result = gck_object_get_handle (GCK_OBJECT (cred));
 		return TRUE;
 	}
@@ -121,14 +124,12 @@ find_unlocked_secret_data (GckCredential *cred, GckObject *object, gpointer user
 {
 	GckSecretCollection *self = GCK_SECRET_COLLECTION (object);
 	GckSecretData **result = user_data;
-	GckSecretData *sdata;
 
 	g_return_val_if_fail (!*result, FALSE);
 
-	sdata = gck_credential_get_data (cred);
-	if (sdata) {
-		g_return_val_if_fail (sdata == self->sdata, FALSE);
-		*result = sdata;
+	*result = gck_credential_pop_data (cred, GCK_TYPE_SECRET_DATA);
+	if (*result) {
+		g_return_val_if_fail (*result == self->sdata, FALSE);
 		return TRUE;
 	}
 
@@ -146,7 +147,7 @@ track_secret_data (GckSecretCollection *self, GckSecretData *data)
 	self->sdata = data;
 	if (self->sdata)
 		g_object_add_weak_pointer (G_OBJECT (self->sdata),
-		                           (gpointer*)&self->sdata);
+		                           (gpointer*)&(self->sdata));
 }
 
 static void
@@ -239,16 +240,19 @@ factory_create_collection (GckSession *session, GckTransaction *transaction,
 	CK_OBJECT_HANDLE handle;
 	CK_ATTRIBUTE *attr;
 	GckManager *manager;
+	GckModule *module;
 	gchar *identifier = NULL;
 	GckSecretData *sdata;
 	gchar *label = NULL;
 	GckCredential *cred;
+	gboolean is_token;
 	CK_RV rv;
 
 	g_return_val_if_fail (GCK_IS_TRANSACTION (transaction), NULL);
 	g_return_val_if_fail (attrs || !n_attrs, NULL);
 
 	manager = gck_manager_for_template (attrs, n_attrs, session);
+	module = gck_session_get_module (session);
 
 	/* Must have a credential, which is not associated with an object yet */
 	if (!gck_attributes_find_ulong (attrs, n_attrs, CKA_G_CREDENTIAL, &handle)) {
@@ -262,17 +266,46 @@ factory_create_collection (GckSession *session, GckTransaction *transaction,
 		return NULL;
 	}
 
-	/* See if a collection attribute was specified, not present means all collections */
+	/* The identifier */
+	attr = gck_attributes_find (attrs, n_attrs, CKA_ID);
+	if (attr != NULL) {
+		gck_attribute_consume (attr);
+		rv = gck_attribute_get_string (attr, &identifier);
+		if (rv != CKR_OK) {
+			gck_transaction_fail (transaction, rv);
+			return NULL;
+		}
+
+		/* Try to find the collection with that identifier */
+		if (!gck_attributes_find_boolean (attrs, n_attrs, CKA_TOKEN, &is_token))
+			collection = gck_secret_collection_find (attr, gck_module_get_manager (module),
+			                                         gck_session_get_manager (session), NULL);
+		else if (is_token)
+			collection = gck_secret_collection_find (attr, gck_module_get_manager (module), NULL);
+		else
+			collection = gck_secret_collection_find (attr, gck_session_get_manager (session), NULL);
+
+		/* Already have one with this identifier? Just return that */
+		if (collection != NULL) {
+			gck_session_complete_object_creation (session, transaction, GCK_OBJECT (collection),
+			                                      FALSE, attrs, n_attrs);
+			return g_object_ref (collection);
+		}
+	}
+
+	/* The label  */
 	attr = gck_attributes_find (attrs, n_attrs, CKA_LABEL);
 	if (attr != NULL) {
+		gck_attribute_consume (attr);
 		rv = gck_attribute_get_string (attr, &label);
 		if (rv != CKR_OK) {
 			gck_transaction_fail (transaction, rv);
 			return NULL;
 		}
-		identifier = g_utf8_strdown (label, -1);
-		g_strdelimit (identifier, ":/\\<>|\t\n\r\v ", '_');
-		gck_attribute_consume (attr);
+
+		/* No identifier? Try to use label */
+		if (identifier == NULL)
+			identifier = g_utf8_strdown (label, -1);
 	}
 
 	if (!identifier || !identifier[0]) {
@@ -280,6 +313,17 @@ factory_create_collection (GckSession *session, GckTransaction *transaction,
 		identifier = g_strdup ("unnamed");
 	}
 
+	if (!label || !label[0]) {
+		g_free (label);
+		if (identifier) {
+			label = g_strdup (identifier);
+		} else {
+			/* TRANSLATORS: This is the label for an keyring created without a label */
+			label = g_strdup (_("Unnamed"));
+		}
+	}
+
+	g_strdelimit (identifier, ":/\\<>|\t\n\r\v ", '_');
 	collection = g_object_new (GCK_TYPE_SECRET_COLLECTION,
 	                           "module", gck_session_get_module (session),
 	                           "identifier", identifier,
@@ -292,11 +336,14 @@ factory_create_collection (GckSession *session, GckTransaction *transaction,
 
 	gck_credential_connect (cred, GCK_OBJECT (collection));
 	sdata = g_object_new (GCK_TYPE_SECRET_DATA, NULL);
-	gck_credential_set_data (cred, sdata, g_object_unref);
+	gck_credential_set_data (cred, GCK_TYPE_SECRET_DATA, sdata);
 	gck_secret_data_set_master (sdata, gck_credential_get_secret (cred));
 	track_secret_data (collection, sdata);
+	g_object_unref (sdata);
 
-	gck_session_complete_object_creation (session, transaction, GCK_OBJECT (collection), attrs, n_attrs);
+	gck_attributes_consume (attrs, n_attrs, CKA_G_CREDENTIAL, G_MAXULONG);
+	gck_session_complete_object_creation (session, transaction, GCK_OBJECT (collection),
+	                                      TRUE, attrs, n_attrs);
 	return GCK_OBJECT (collection);
 }
 
@@ -327,15 +374,17 @@ change_master_password (GckSecretCollection *self, GckTransaction *transaction,
 	g_assert (GCK_IS_TRANSACTION (transaction));
 	g_assert (GCK_IS_CREDENTIAL (cred));
 
-	if (!self->sdata)
-		return gck_transaction_fail (transaction, CKR_USER_NOT_LOGGED_IN);
+	if (!self->sdata) {
+		gck_transaction_fail (transaction, CKR_USER_NOT_LOGGED_IN);
+		return;
+	}
 
 	previous = gck_secret_data_get_master (self->sdata);
 	if (previous != NULL)
 		g_object_ref (previous);
 
 	gck_credential_connect (cred, GCK_OBJECT (self));
-	gck_credential_set_data (cred, g_object_ref (self->sdata), g_object_unref);
+	gck_credential_set_data (cred, GCK_TYPE_SECRET_DATA, self->sdata);
 	gck_secret_data_set_master (self->sdata, gck_credential_get_secret (cred));
 
 	gck_transaction_add (transaction, self, complete_master_password, previous);
@@ -349,15 +398,12 @@ static CK_RV
 gck_secret_collection_get_attribute (GckObject *base, GckSession *session, CK_ATTRIBUTE_PTR attr)
 {
 	GckSecretCollection *self = GCK_SECRET_COLLECTION (base);
-	CK_OBJECT_HANDLE handle = 0;
 
 	switch (attr->type) {
 	case CKA_CLASS:
 		return gck_attribute_set_ulong (attr, CKO_G_COLLECTION);
-	case CKA_G_CREDENTIAL:
-		gck_session_for_each_credential (session, GCK_OBJECT (self),
-		                                 find_unlocked_credential, &handle);
-		return gck_attribute_set_ulong (attr, handle);
+	case CKA_G_CREDENTIAL_TEMPLATE:
+		return gck_attribute_set_template (attr, self->template);
 	}
 	return GCK_OBJECT_CLASS (gck_secret_collection_parent_class)->get_attribute (base, session, attr);
 }
@@ -369,25 +415,41 @@ gck_secret_collection_set_attribute (GckObject *object, GckSession *session,
 	GckSecretCollection *self = GCK_SECRET_COLLECTION (object);
 	CK_OBJECT_HANDLE handle = 0;
 	GckCredential *cred;
+	GArray *template;
 	CK_RV rv;
 
 	switch (attr->type) {
 	case CKA_G_CREDENTIAL:
-		gck_session_for_each_credential (session, GCK_OBJECT (self),
-		                                 find_unlocked_credential, &handle);
-		if (handle == 0)
-			return gck_transaction_fail (transaction, CKR_USER_NOT_LOGGED_IN);
+		gck_credential_for_each (session, GCK_OBJECT (self),
+		                         find_unlocked_credential, &handle);
+		if (handle == 0) {
+			gck_transaction_fail (transaction, CKR_USER_NOT_LOGGED_IN);
+			return;
+		}
 		rv = gck_attribute_get_ulong (attr, &handle);
-		if (rv != CKR_OK)
-			return gck_transaction_fail (transaction, rv);
+		if (rv != CKR_OK) {
+			gck_transaction_fail (transaction, rv);
+			return;
+		}
 		cred = lookup_unassociated_credential (session, handle);
-		if (cred == NULL)
-			return gck_transaction_fail (transaction, CKR_ATTRIBUTE_VALUE_INVALID);
+		if (cred == NULL) {
+			gck_transaction_fail (transaction, CKR_ATTRIBUTE_VALUE_INVALID);
+			return;
+		}
 		change_master_password (self, transaction, cred);
 		return;
+	case CKA_G_CREDENTIAL_TEMPLATE:
+		rv = gck_attribute_get_template (attr, &template);
+		if (rv != CKR_OK) {
+			gck_transaction_fail (transaction, rv);
+			return;
+		}
+		gck_template_free (self->template);
+		self->template = template;
+		return;
 	};
 
-	return GCK_OBJECT_CLASS (gck_secret_collection_parent_class)->set_attribute (object, session, transaction, attr);
+	GCK_OBJECT_CLASS (gck_secret_collection_parent_class)->set_attribute (object, session, transaction, attr);
 }
 
 static CK_RV
@@ -397,6 +459,7 @@ gck_secret_collection_real_unlock (GckObject *obj, GckCredential *cred)
 	GckDataResult res;
 	GckSecretData *sdata;
 	GckSecret *master;
+	CK_RV rv;
 
 	master = gck_credential_get_secret (cred);
 
@@ -406,7 +469,7 @@ gck_secret_collection_real_unlock (GckObject *obj, GckCredential *cred)
 			return CKR_PIN_INCORRECT;
 
 		/* Credential now tracks our secret data */
-		gck_credential_set_data (cred, g_object_ref (self->sdata), g_object_unref);
+		gck_credential_set_data (cred, GCK_TYPE_SECRET_DATA, self->sdata);
 		return CKR_OK;
 	}
 
@@ -428,23 +491,27 @@ gck_secret_collection_real_unlock (GckObject *obj, GckCredential *cred)
 
 	switch (res) {
 	case GCK_DATA_SUCCESS:
-		gck_credential_set_data (cred, sdata, g_object_unref);
+		gck_credential_set_data (cred, GCK_TYPE_SECRET_DATA, sdata);
 		track_secret_data (self, sdata);
-		return CKR_OK;
+		rv = CKR_OK;
+		break;
 	case GCK_DATA_LOCKED:
-		g_object_unref (sdata);
-		return CKR_PIN_INCORRECT;
+		rv = CKR_PIN_INCORRECT;
+		break;
 	case GCK_DATA_UNRECOGNIZED:
-		g_object_unref (sdata);
 		g_message ("unrecognized or invalid keyring: %s", self->filename);
-		return CKR_FUNCTION_FAILED;
+		rv = CKR_FUNCTION_FAILED;
+		break;
 	case GCK_DATA_FAILURE:
-		g_object_unref (sdata);
 		g_message ("failed to read or parse keyring: %s", self->filename);
-		return CKR_GENERAL_ERROR;
+		rv = CKR_GENERAL_ERROR;
+		break;
 	default:
 		g_assert_not_reached ();
 	}
+
+	g_object_unref (sdata);
+	return rv;
 }
 
 static void
@@ -458,15 +525,25 @@ static gboolean
 gck_secret_collection_real_is_locked (GckSecretObject *obj, GckSession *session)
 {
 	GckSecretCollection *self = GCK_SECRET_COLLECTION (obj);
-	return gck_secret_collection_unlocked_data (self, session) ? FALSE : TRUE;
+	return !gck_secret_collection_unlocked_have (self, session);
 }
 
 static void
 gck_secret_collection_init (GckSecretCollection *self)
 {
-	self->items = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
-}
+	CK_ULONG idle = 0;
+	CK_ULONG after = 0;
+	CK_BBOOL token = CK_TRUE;
+	CK_ATTRIBUTE attrs[] = {
+		{ CKA_TOKEN, &token, sizeof (token) },
+		{ CKA_GNOME_TRANSIENT, &token, sizeof (token) },
+		{ CKA_G_DESTRUCT_IDLE, &idle, sizeof (idle) },
+		{ CKA_G_DESTRUCT_AFTER, &after, sizeof (after) },
+	};
 
+	self->items = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
+	self->template = gck_template_new (attrs, G_N_ELEMENTS (attrs));
+}
 
 static void
 gck_secret_collection_set_property (GObject *obj, guint prop_id, const GValue *value,
@@ -524,6 +601,9 @@ gck_secret_collection_finalize (GObject *obj)
 	g_free (self->filename);
 	self->filename = NULL;
 
+	gck_template_free (self->template);
+	self->template = NULL;
+
 	G_OBJECT_CLASS (gck_secret_collection_parent_class)->finalize (obj);
 }
 
@@ -727,8 +807,26 @@ gck_secret_collection_destroy_item (GckSecretCollection *self, GckTransaction *t
 	remove_item (self, transaction, item);
 }
 
+gboolean
+gck_secret_collection_unlocked_have (GckSecretCollection *self, GckSession *session)
+{
+	CK_OBJECT_HANDLE handle = 0;
+
+	g_return_val_if_fail (GCK_IS_SECRET_COLLECTION (self), FALSE);
+	g_return_val_if_fail (GCK_IS_SESSION (session), FALSE);
+
+	/*
+	 * Look for credential objects that this session has access
+	 * to, and use those to find the secret data. If a secret data is
+	 * found, it should match the one we are tracking in self->sdata.
+	 */
+
+	gck_credential_for_each (session, GCK_OBJECT (self), find_unlocked_credential, &handle);
+	return handle != 0;
+}
+
 GckSecretData*
-gck_secret_collection_unlocked_data (GckSecretCollection *self, GckSession *session)
+gck_secret_collection_unlocked_use (GckSecretCollection *self, GckSession *session)
 {
 	GckSecretData *sdata = NULL;
 
@@ -741,8 +839,8 @@ gck_secret_collection_unlocked_data (GckSecretCollection *self, GckSession *sess
 	 * found, it should match the one we are tracking in self->sdata.
 	 */
 
-	gck_session_for_each_credential (session, GCK_OBJECT (self),
-	                                 find_unlocked_secret_data, &sdata);
+	gck_credential_for_each (session, GCK_OBJECT (self),
+	                         find_unlocked_secret_data, &sdata);
 
 	return sdata;
 }
@@ -784,8 +882,10 @@ gck_secret_collection_save (GckSecretCollection *self, GckTransaction *transacti
 	g_return_if_fail (!gck_transaction_get_failed (transaction));
 
 	/* HACK: We can't save unless the secret data was loaded */
-	if (!self->sdata)
-		return gck_transaction_fail (transaction, CKR_USER_NOT_LOGGED_IN);
+	if (!self->sdata) {
+		gck_transaction_fail (transaction, CKR_USER_NOT_LOGGED_IN);
+		return;
+	}
 
 	/* Don't save ourselves if no filename */
 	if (!self->filename)
@@ -827,3 +927,41 @@ gck_secret_collection_destroy (GckSecretCollection *self, GckTransaction *transa
 	if (self->filename)
 		gck_transaction_remove_file (transaction, self->filename);
 }
+
+gint
+gck_secret_collection_get_lock_idle (GckSecretCollection *self)
+{
+	gulong value;
+	g_return_val_if_fail (GCK_IS_SECRET_COLLECTION (self), 0);
+	if (!gck_template_find_ulong (self->template, CKA_G_DESTRUCT_IDLE, &value))
+		value = 0;
+	return (gint)value;
+}
+
+void
+gck_secret_collection_set_lock_idle (GckSecretCollection *self, gint lock_timeout)
+{
+	CK_ULONG value = (lock_timeout < 0) ? 0 : lock_timeout;
+	CK_ATTRIBUTE attr = { CKA_G_DESTRUCT_IDLE, &value, sizeof (value) };
+	g_return_if_fail (GCK_IS_SECRET_COLLECTION (self));
+	gck_template_set (self->template, &attr);
+}
+
+gint
+gck_secret_collection_get_lock_after (GckSecretCollection *self)
+{
+	gulong value;
+	g_return_val_if_fail (GCK_IS_SECRET_COLLECTION (self), 0);
+	if (!gck_template_find_ulong (self->template, CKA_G_DESTRUCT_AFTER, &value))
+		value = 0;
+	return (gint)value;
+}
+
+void
+gck_secret_collection_set_lock_after (GckSecretCollection *self, gint lock_timeout)
+{
+	CK_ULONG value = (lock_timeout < 0) ? 0 : lock_timeout;
+	CK_ATTRIBUTE attr = { CKA_G_DESTRUCT_AFTER, &value, sizeof (value) };
+	g_return_if_fail (GCK_IS_SECRET_COLLECTION (self));
+	gck_template_set (self->template, &attr);
+}
